%   Diese MATLAB Funktion (Callback) liest die RS232 Daten des Arduino und
%   speichert Sie im Handle unter UserData
%
%   SYNTAX:
%     leseArduinoDaten(src, ~)
%
%   BESCHREIBUNG:
%   Mit dem Aufruf
%     configureCallback(hArduinoObj,"terminator",@leseArduinoDaten);
%   wir bei jedem im seriellen Datenstrom gefundenen Terminator (z. B.
%   Carrier Return/Line Feed) diese Funktion aufgerufen.
%
%   Eingabewerte sind:
%     src: Objekthandle zur seriellen Vebindung
%
%   Rückgabewert ist:
%     keiner. die Daten werden in der Objektstruktur gespeichert
%     src: Objekthandle zur seriellen Vebindung
%
%   BEISPIEL:
%     configureCallback(hArduinoObj,"terminator",@leseArduinoDaten);

%****************************************************************
% Modul	          : leseArduinoDaten.m                          *
%                                                               *
% Datum           : 27. März 2021                               *
%                                                               *
% Funktion        : Die vom Alphabot gemessene Temperatur wird  *
%                   aus dem RS232 ASCII Strom gelesen und mit   *
%                   Zeitstempel und Zähler gespeichert und      *
%                   nach 1001 Messwerten visualisiert.          *
%                                                               *
% Implementation  : MATLAB R2020b                               *
%                                                               *
% Author          : (c) 2021 Prof. Schneider                    *
%                   Alle Rechte vorbehalten.                    *
%                                                               *
% Bemerkung       :                                             *
%                                                               *
% Letzte Änderung : 27. März 2021                               *
%                                                               *
%****************************************************************

% Quelle: https://www.mathworks.com/help/matlab/import_export/read-streaming-data-from-arduino.html *

function leseArduinoDaten(src, ~)
persistent t
if isempty(t) % einmalige Initialisierung
    tic % Uhr einschalten
end
t=toc; % Zeitmessung

% Lesen der ASCII Daten aus dem Objekt des seriellen Ports.
data = readline(src);

% Der String wird in einen numerischen Datentyp konvertiert und in UserData
% des Objekts des seriellen Ports gespeichert.
src.UserData.Data(end+1) = str2double(data);

% Der Zähler für die im Objekt des seriellen Ports gespeicherten Daten wird inkrementiert.
src.UserData.Count = src.UserData.Count + 1;

% Die Zeit des Nachrichtenempfangs wird ebenfalls gespeichert.
src.UserData.Zeit(end+1) = t;


% In src.UserData.nDatenEnde wurde die Abbruchbedingung 1001 gespeichert.
% Wurden >1001 Daten vom Arduino gesammelt wird das Callback ausgeschaltet
% und die Daten werden visualisiert.
if src.UserData.Count > src.UserData.nDatenEnde
    configureCallback(src, "off"); % Callback ausschalten
    %% Plot der Ergebnisses
    aZeit = src.UserData.Zeit;
    aTemp = src.UserData.Data;
    plot(aZeit,aTemp,'b.-');
    xlabel('Zeit in s')
    ylabel('Temperatur in °C')
    
    disp('.. Serieller Monitor beendet!')
end
end